@@ -26,9 +26,10 @@ import subprocess
 import tempfile
 import os
 import re
+from typing import Optional, Tuple, Iterator
 
 from debian import deb822
-from debian.changelog import Changelog, ChangelogParseError
+from debian.changelog import Changelog, ChangelogParseError, Version
 from debian.copyright import Copyright, NotMachineReadableError
 
 from debmutate.changelog import (
@@ -46,6 +47,7 @@ from ... import (
     osutils,
     urlutils,
     )
+from ...transport import NoSuchFile
 from ...export import export
 from ...trace import (
     mutter,
@@ -123,24 +125,6 @@ DEBIAN_POCKETS = ('', '-security', '-proposed-updates', '-backports')
 UBUNTU_POCKETS = ('', '-proposed', '-updates', '-security', '-backports')
 
 
-def safe_decode(s):
-    """Decode a string into a Unicode value."""
-    if isinstance(s, str):  # Already unicode
-        mutter('safe_decode() called on an already-decoded string: %r' % (s,))
-        return s
-    try:
-        return s.decode('utf-8')
-    except UnicodeDecodeError as e:
-        mutter('safe_decode(%r) falling back to iso-8859-1: %s' % (s, e))
-        # TODO: Looking at BeautifulSoup it seems to use 'chardet' to try to
-        #       guess the encoding of a given text stream. We might want to
-        #       take a closer look at that.
-        # TODO: Another possibility would be to make the fallback encoding
-        #       configurable, possibly exposed as a command-line flag, for now,
-        #       this seems 'good enough'.
-        return s.decode('iso-8859-1')
-
-
 def recursive_copy(fromdir, todir):
     """Copy the contents of fromdir to todir.
 
@@ -218,9 +202,9 @@ def find_changelog(t, subpath='', merge=False, max_blocks=1, strict=False):
             # "." then it will have found debian/changelog. Try and detect
             # this.
             debian_file = osutils.pathjoin(subpath, 'debian')
-            if (t.is_versioned(debian_file) and
-                    t.kind(debian_file) == 'symlink' and
-                    t.get_symlink_target(debian_file) == '.'):
+            if (t.is_versioned(debian_file)
+                    and t.kind(debian_file) == 'symlink'
+                    and t.get_symlink_target(debian_file) == '.'):
                 changelog_file = 'changelog'
                 top_level = True
         mutter("Using '%s' to get package information", changelog_file)
@@ -246,10 +230,10 @@ def tarball_name(package, version, component=None, format=None):
     """
     if format is None:
         format = 'gz'
-    name = "%s_%s.orig" % (package, str(version))
+    name = "{}_{}.orig".format(package, str(version))
     if component is not None:
         name += "-" + component
-    return "%s.tar.%s" % (name, format)
+    return "{}.tar.{}".format(name, format)
 
 
 def suite_to_distribution(suite):
@@ -488,12 +472,12 @@ def get_commit_info_from_changelog(changelog, branch, _lplib=None):
     bugs = []
     if changelog._blocks:
         block = changelog._blocks[0]
-        authors = [safe_decode(block.author)]
+        authors = [block.author]
         authors += find_extra_authors(block.changes())
         changes = strip_changelog_message(block.changes())
         bugs = find_bugs_fixed(block.changes(), branch, _lplib=_lplib)
         thanks = find_thanks(block.changes())
-        message = safe_decode("\n".join(changes).replace("\r", ""))
+        message = "\n".join(changes).replace("\r", "")
     return (message, authors, thanks, bugs)
 
 
@@ -568,9 +552,9 @@ def find_previous_upload(tree, subpath, merge=False):
     Ubuntu.
     """
     try:
-        cl, top_level = find_changelog(tree, subpath, merge, max_blocks=None)
-    except ChangelogParseError:
-        raise UnableToFindPreviousUpload()
+        cl, _top_level = find_changelog(tree, subpath, merge, max_blocks=None)
+    except ChangelogParseError as ex:
+        raise UnableToFindPreviousUpload() from ex
     return changelog_find_previous_upload(cl)
 
 
@@ -617,9 +601,9 @@ def tree_contains_upstream_source(tree, subpath=''):
     :return: Boolean indicating whether or not the tree contains the upstream
         source. None if the tree is empty
     """
-    present_files = set(
-        [f[0] for f in tree.list_files(recursive=False, from_dir=subpath)
-         if f[1] == 'V'])
+    present_files = {
+        f[0] for f in tree.list_files(recursive=False, from_dir=subpath)
+        if f[1] == 'V'}
     if len(present_files) == 0:
         return None
     packaging_files = frozenset([
@@ -636,11 +620,11 @@ def tree_get_source_format(tree, subpath=''):
     filename = osutils.pathjoin(subpath, "debian/source/format")
     try:
         text = tree.get_file_text(filename)
-    except IOError as e:
-        if e.errno == errno.ENOENT:
+    except OSError as ex:
+        if ex.errno == errno.ENOENT:
             return FORMAT_1_0
         raise
-    except errors.NoSuchFile:
+    except NoSuchFile:
         return FORMAT_1_0
     return text.strip().decode('ascii')
 
@@ -655,10 +639,11 @@ NORMAL_SOURCE_FORMATS = [FORMAT_3_0_QUILT]
 
 class InconsistentSourceFormatError(BzrError):
 
-    _fmt = ("Inconsistency between source format and version: version is "
-            "%(version_bool)snative, format is %(format_bool)snative.")
+    _fmt = ("Inconsistency between source format and version: "
+            "version %(version)s is %(version_bool)snative, "
+            "format '%(format)s' is %(format_bool)snative.")
 
-    def __init__(self, version_native, format_native):
+    def __init__(self, version_native, format_native, version_str, format_str):
         if version_native:
             version_bool = ""
         else:
@@ -668,7 +653,8 @@ class InconsistentSourceFormatError(BzrError):
         else:
             format_bool = "not "
         BzrError.__init__(
-            self, version_bool=version_bool, format_bool=format_bool)
+            self, version_bool=version_bool, format_bool=format_bool,
+            version=version_str, format=format_str)
 
 
 def guess_build_type(tree, version, subpath='', contains_upstream_source=True):
@@ -697,11 +683,13 @@ def guess_build_type(tree, version, subpath='', contains_upstream_source=True):
     # native, but it *could* be native.
     if type(version_native) is bool and type(format_native) is bool:
         if version_native is True and format_native is False:
-            raise InconsistentSourceFormatError(version_native, format_native)
+            raise InconsistentSourceFormatError(
+                version_native, format_native, version, source_format)
         if version_native is False and format_native is True:
             warning(
-                'Version suggests non-native package, '
-                'but format is for native.')
+                'Version (%s) suggests non-native package, '
+                'but format (%s) is for native.',
+                version, source_format)
 
     if version_native or format_native:
         return BUILD_TYPE_NATIVE
@@ -714,11 +702,11 @@ def guess_build_type(tree, version, subpath='', contains_upstream_source=True):
 
 def component_from_orig_tarball(tarball_filename, package, version):
     tarball_filename = os.path.basename(tarball_filename)
-    prefix = "%s_%s.orig" % (package, version)
+    prefix = f"{package}_{version}.orig"
     if not tarball_filename.startswith(prefix):
         raise ValueError(
-            "invalid orig tarball file %s does not have expected prefix %s" % (
-                tarball_filename, prefix))
+            f"invalid orig tarball file {tarball_filename} "
+            f"does not have expected prefix {prefix}")
     base = tarball_filename[len(prefix):]
     for ext in (".tar.gz", ".tar.bz2", ".tar.lzma", ".tar.xz"):
         if tarball_filename.endswith(ext):
@@ -726,7 +714,7 @@ def component_from_orig_tarball(tarball_filename, package, version):
             break
     else:
         raise ValueError(
-            "orig tarball file %s has unknown extension" % tarball_filename)
+            f"orig tarball file {tarball_filename} has unknown extension")
     if base == "":
         return None
     elif base[0] == "-":
@@ -734,8 +722,7 @@ def component_from_orig_tarball(tarball_filename, package, version):
         return base[1:]
     else:
         raise ValueError(
-                "Invalid extra characters in tarball filename %s" %
-                tarball_filename)
+            f"Invalid extra characters in tarball filename {tarball_filename}")
 
 
 class TarFailed(BzrError):
@@ -755,7 +742,7 @@ def needs_strip_components(tf):
 
 
 def extract_orig_tarball(tarball_filename, component, target,
-                         strip_components=None):
+                         strip_components: Optional[bool] = None) -> None:
     """Extract an orig tarball.
 
     :param tarball: Path to the tarball
@@ -767,8 +754,8 @@ def extract_orig_tarball(tarball_filename, component, target,
     if tarball_filename.endswith(".tar.bz2"):
         tar_args.append('xjf')
         tf = TarFile.bz2open(tarball_filename)
-    elif (tarball_filename.endswith(".tar.lzma") or
-          tarball_filename.endswith(".tar.xz")):
+    elif (tarball_filename.endswith(".tar.lzma")
+            or tarball_filename.endswith(".tar.xz")):
         tar_args.append('xJf')
         tf = TarFile.xzopen(tarball_filename)
     elif tarball_filename.endswith(".tar"):
@@ -819,17 +806,18 @@ def extract_orig_tarballs(tarballs, target, strip_components=None):
             strip_components=strip_components)
 
 
-def dput_changes(path):
+def dput_changes(path: str) -> None:
     """Upload a package."""
     (bd, changes_file) = os.path.split(path)
     subprocess.check_call(["dput", changes_file], cwd=bd)
 
 
-def find_changes_files(path, package, version):
+def find_changes_files(path: str, package: str, version: Version) -> Iterator[Tuple[str, os.DirEntry]]:
     non_epoch_version = version.upstream_version
     if version.debian_version is not None:
         non_epoch_version += "-%s" % version.debian_version
-    c = re.compile('%s_%s_(.*).changes' % (re.escape(package), re.escape(non_epoch_version)))
+    c = re.compile('{}_{}_(.*).changes'.format(
+        re.escape(package), re.escape(non_epoch_version)))
     for entry in os.scandir(path):
         m = c.match(entry.name)
         if m:
@@ -890,13 +878,17 @@ def detect_version_kind(upstream_version):
 def export_with_nested(tree, dest, **kwargs):
     with tree.lock_read():
         try:
-            try:
-                return export(tree, dest, recurse_nested=True, **kwargs)
-            except TypeError:  # brz < 3.3
-                if any(tree.iter_references()):
-                    raise NotImplementedError('unable to export tree references')
-                return export(tree, dest, **kwargs)
+            return export(tree, dest, recurse_nested=True, **kwargs)
         except BaseException:
             if os.path.isfile(dest):
                 os.unlink(dest)
             raise
+
+
+def debsign(path: str, keyid: Optional[str] = None) -> None:
+    (bd, changes_file) = os.path.split(path)
+    args = ["debsign"]
+    if keyid:
+        args.append("-k%s" % keyid)
+    args.append(changes_file)
+    subprocess.check_call(args, cwd=bd)
